import createDecorator from '@loafer/core/util/decorators/createDecorator';
import ConfigurationDefinition from '@loafer/context/definition/ConfigurationDefinition';
import { getInjectableDefinition } from '@loafer/pouches/util/metadata';

import GetAppContext from '@loafer/context';
import { InjectableSterotype } from '@loafer/pouches/constants';
import DefaultPouchFactory from '@loafer/pouches/factory/implement/DefaultPouchFactory';
import PouchDefinitionRegistry from '@loafer/pouches/factory/registry/PouchDefinitionRegistry';
import { validateQualifier } from '@loafer/pouches/util/qualifier';

const Configuration = (qualifier?: string | symbol ) => createDecorator('Configuration', {
  classDecorator: <TFunction extends Function>(target: TFunction): TFunction | void => {
    let configurationDefinition: ConfigurationDefinition = getInjectableDefinition(target.prototype, ConfigurationDefinition, false);
    if (undefined !== configurationDefinition) {
      throw new Error('Cannot apply @Injectable or @Injectable stereotype decorator multiple times.');
    }

    configurationDefinition = getInjectableDefinition(target.prototype, ConfigurationDefinition, true);
    configurationDefinition.Stereotype = InjectableSterotype.CONFIGURATION;
    configurationDefinition.Qualifier = validateQualifier(target.prototype, qualifier);

    return target;
  },

});


export default Configuration;
