import createDecorator from '@loafer/core/util/decorators/createDecorator';
import InjectableDefinition from '@loafer/pouches/definition/InjectableDefinition';
import { getInjectableDefinition } from '@loafer/pouches/util/metadata';

import GetAppContext from '@loafer/context';
import { InjectableSterotype } from '@loafer/pouches/constants';
import { validateQualifier } from '@loafer/pouches/util/qualifier';
import DefaultPouchFactory from '@loafer/pouches/factory/implement/DefaultPouchFactory';
import PouchDefinitionRegistry from '@loafer/pouches/factory/registry/PouchDefinitionRegistry';

const Service = (qualifier?: string | symbol ) => createDecorator('Service', {
  classDecorator: <TFunction extends Function>(target: TFunction): TFunction | void => {
    let injectableDefinition: InjectableDefinition = getInjectableDefinition(target.prototype, InjectableDefinition, false);
    if (undefined !== injectableDefinition) {
      throw new Error('Cannot apply @Injectable or @Injectable stereotype decorator multiple times.');
    }

    injectableDefinition = getInjectableDefinition(target.prototype, InjectableDefinition, true);
    injectableDefinition.Stereotype = InjectableSterotype.SERVICE;
    injectableDefinition.Qualifier = validateQualifier(target.prototype, qualifier);

    return target;
  },

});


export default Service;
